package com.develou.snackbar_en_android

import android.os.Bundle
import android.view.KeyEvent
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import com.google.android.material.floatingactionbutton.FloatingActionButton
import com.google.android.material.snackbar.BaseTransientBottomBar
import com.google.android.material.snackbar.Snackbar
import com.google.android.material.switchmaterial.SwitchMaterial
import com.google.android.material.textfield.TextInputLayout

class MainActivity : AppCompatActivity() {
    private var useStyle: Boolean = false

    private val logs = mutableListOf<String>()
    private val saveCommand = SaveLogCommand(logs)

    private var counter = 0

    private lateinit var logTextField: TextInputLayout
    private lateinit var counterText: TextView

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        setUpSwitch()
        setUpEditText()
        setUpFab()
        setUpCounterText()
    }

    private fun setUpSwitch() {
        val switch: SwitchMaterial = findViewById(R.id.style_switch)
        switch.setOnCheckedChangeListener { _, isChecked ->
            useStyle = isChecked
        }
    }

    private fun setUpEditText() {
        logTextField = findViewById(R.id.log_text_field)
        logTextField.editText?.setOnEditorActionListener { _, actionId, _ ->
            return@setOnEditorActionListener when (actionId) {
                EditorInfo.IME_ACTION_DONE -> {
                    saveLog()
                    true
                }
                else -> false
            }
        }
    }

    private fun saveLog() {
        val log = getLogText()

        saveCommand.saveLog(log)

        updateLogsList()
        hideKeyboard()
        buildSnackbar()
    }

    private fun getLogText(): String {
        return logTextField.editText?.text.toString()
    }

    private fun updateLogsList() {
        val logView: TextView = findViewById(R.id.log_view)
        logView.text = logs.joinToString(separator = "\n")
    }

    private fun hideKeyboard() {
        val imm = getSystemService(INPUT_METHOD_SERVICE)
                as InputMethodManager
        imm.hideSoftInputFromWindow(logTextField.windowToken, 0)
    }

    private fun buildSnackbar() {
        val snackbar = Snackbar.make(
            findViewById(R.id.constraint_layout),
            "Registro guardado",
            BaseTransientBottomBar.LENGTH_LONG
        ).setAction("Deshacer") {
            saveCommand.undo()
            updateLogsList()
        }
        snackbar.setAnchorView(R.id.save_fab)
        snackbar.addCallback(object : Snackbar.Callback() {
            override fun onDismissed(transientBottomBar: Snackbar?, event: Int) {
                if (event == DISMISS_EVENT_ACTION) {
                    counter++
                    updateTextCounter()
                }
            }
        })
        if (useStyle) {
            snackbar.setBackgroundTint(color(R.color.purple_700))
            snackbar.setActionTextColor(color(R.color.purple_200))
        } else {
            snackbar.setBackgroundTint(color(R.color.teal_700))
            snackbar.setActionTextColor(color(R.color.teal_200))

        }
        snackbar.show()
    }

    private fun updateTextCounter() {
        counterText.text = "Cantidad de inserciones deshecha: $counter"
    }

    private fun color(color: Int) = ContextCompat.getColor(this, color)

    private fun setUpFab() {
        val saveFab: FloatingActionButton = findViewById(R.id.save_fab)
        saveFab.setOnClickListener {
            saveLog()
        }
    }

    private fun setUpCounterText() {
        counterText = findViewById(R.id.text_counter)
        updateTextCounter()
    }
}

